# coding=utf-8
"""
Tests Unit objects
"""

import pytest

from emiz.mission import BaseUnit
from emiz.miz import Miz


def test_units(mission):
    value = 0
    for unit in mission.blue_coa.units:
        value += 1
        assert isinstance(unit, BaseUnit)
    assert value == 3
    value = 0
    for unit in mission.red_coa.units:
        value += 1
        assert isinstance(unit, BaseUnit)
    assert value == 33


def test_get_units_from_category(mission, all_objects):
    for invalid_category in ('caribou', 'Plane', 'plAne', 'ships', -1, 0, 1, True, False, None):
        with pytest.raises(ValueError):
            for _ in mission.blue_coa.get_units_from_category(invalid_category):
                pass
    with Miz(all_objects) as miz:
        # for unit in mission.blue_coa.units:
        #     print(unit.group_category)
        for category in ('ship', 'plane', 'helicopter', 'vehicle'):
            value = 0
            for unit in miz.mission._blue_coa.get_units_from_category(category):
                assert isinstance(unit, BaseUnit)
                assert unit.group_category == category
                value += 1
            assert value == 1


def test_get_unit_by_id(mission):
    for unit_id in range(1, 3):
        assert isinstance(mission.blue_coa.get_unit_by_id(unit_id), BaseUnit)
    for unit_id in range(9, 41):
        assert isinstance(mission.red_coa.get_unit_by_id(unit_id), BaseUnit)
    for wrong_unit_id in (-1, False, None, True, 'caribou'):
        with pytest.raises(ValueError, msg=wrong_unit_id):
            mission.red_coa.get_unit_by_id(wrong_unit_id)
    for non_existing_unit_id in (42, 50, 100, 150, mission.next_unit_id):
        assert mission.red_coa.get_group_by_id(non_existing_unit_id) is None
        assert mission.blue_coa.get_group_by_id(non_existing_unit_id) is None


def test_unit_by_name(mission):
    for unit_name in ('etcher', 'gal', 'gilles'):
        assert isinstance(mission.blue_coa.get_unit_by_name(unit_name), BaseUnit)
    for i in range(1, 33):
        assert isinstance(mission.red_coa.get_unit_by_name('Unit #{}'.format(str(i).zfill(3))), BaseUnit)
    for wrong_unit_name in (-1, 0, 1, False, True, None):
        with pytest.raises(ValueError, msg=wrong_unit_name):
            mission.blue_coa.get_unit_by_name(wrong_unit_name)
    for non_existing_unit_name in ('yup', 'yop', 'unit #017', 'Gilles', 'etcheR'):
        assert mission.red_coa.get_unit_by_name(non_existing_unit_name) is None
        assert mission.blue_coa.get_unit_by_name(non_existing_unit_name) is None


def test_get_unit_by_name(mission):
    unit = mission.get_unit_by_name('etcher')
    assert isinstance(unit, BaseUnit)
    assert unit.unit_id == 1
    assert mission.get_unit_by_name('le_caribou_puissant') is None



# coding=utf-8
"""
Stupid class I wrote a while back to validate values (dummy me)
"""
from os.path import exists
from re import fullmatch as re_full_match


# pylint: disable=too-many-instance-attributes
class Validator:
    """Validates many kind of values against pre-defined conditions, raises Exception and logs errors"""

    def __init__(
            self,
            *,
            _type=None,
            _instance=None,
            _min=None,
            _max=None,
            _regex=None,
            _in_list=None,
            _path_exists=False,
            exc=None,
            logger=None
    ):
        self.type = _type
        self.instance = _instance
        self.min = _min
        self.max = _max
        self.regex = _regex
        self.in_list = _in_list
        self.path_exists = _path_exists
        self.exc = exc or ValueError
        self.logger = logger

    def validate(self, value, param_name, exc=None, logger=None):
        """
        :param value: value to validate
        :param param_name: name of the value (for logging purpose)
        :param exc: exception to raise (default is "ValidatorError")
        :param logger: logger to use (default will be "Validator.logger")
        """
        if exc is not None:
            self.exc = exc

        if logger is not None:
            self.logger = logger

        if self.type is not None and not type(value) == self.type:  # pylint: disable=unidiomatic-typecheck
            self.error(
                f'invalid type for parameter "{param_name}": {type(value)} (value: {value}) -- expected {self.type}'
            )

        if self.instance is not None and not isinstance(value, self.instance):
            self.error(
                f'invalid instance for parameter "{param_name}": {type(value)} (value: {value})'
                f' -- expected {self.instance}'
            )

        if self.min is not None and value < self.min:
            self.error(f'invalid value for parameter "{param_name}" (under minima of {self.min}): {value}')

        if self.max is not None and value > self.max:
            self.error(f'invalid value for parameter "{param_name}" (over maxima if {self.max}): {value}')

        if self.regex is not None and not re_full_match(self.regex, value):
            self.error(f'invalid value for parameter "{param_name}" (should match: "{self.regex}"): {value}')

        if self.in_list is not None and value not in self.in_list:
            self.error(f'invalid value for parameter "{param_name}"; "{value}" is not in list: {self.in_list}')

        if self.path_exists and not exists(value):
            self.error(f'"{param_name}" file does not exist: {value}')

        return True

    def error(self, error_msg):
        """
        Outputs error message on own logger. Also raises exceptions if need be.

        Args:
            error_msg: message to output

        """
        if self.logger is not None:
            self.logger.error(error_msg)

        if self.exc is not None:
            raise self.exc(error_msg)


VALID_BOOL = Validator(_type=bool)
VALID_STR = Validator(_type=str)
VALID_INT = Validator(_type=int)
VALID_POSITIVE_INT = Validator(_type=int, _min=0)
VALID_NEGATIVE_INT = Validator(_type=int, _max=0)
VALID_FLOAT = Validator(_type=float)
VALID_EXISTING_PATH = Validator(_path_exists=True)
VALID_LIST = Validator(_type=list)
VALID_DICT = Validator(_type=dict)
NOT_A_STR = [-1, True, False, None, 1234, dict(), list(), set(), tuple()]
NOT_AN_INT = [True, False, None, '', 'meh', 12.34, dict(), list(), set(), tuple()]
NOT_A_POSITIVE_INT = [True, False, None, '', 'meh', 12.34, -1, -100000, dict(), list(), set(), tuple()]
NOT_A_BOOL = [None, 1, 12.34, 'meh', dict(), list(), set(), tuple()]



# coding=utf-8
"""
Query METAR and TAF information from http://tgftp.nws.noaa.gov
"""
import typing

import requests

_BASE_TAF_URL = r'http://tgftp.nws.noaa.gov/data/forecasts/taf/stations/{station}.TXT'
_BASE_METAR_URL = r'http://tgftp.nws.noaa.gov/data/observations/metar/stations/{station}.TXT'


def retrieve_taf(station_icao) -> typing.Tuple[typing.Union[str, None], typing.Union[str, None]]:
    """
    Retrieves a TAF string from an online database

    Args:
        station_icao: ICAO of the station

    Returns:
        tuple of error, metar_str
    """
    url = _BASE_TAF_URL.format(station=station_icao)
    with requests.get(url) as resp:
        if not resp.ok:
            return f'unable to obtain TAF for station {station_icao}\n' \
                   f'Got to "http://tgftp.nws.noaa.gov/data/observations/metar/stations" ' \
                   f'for a list of valid stations', None
        return None, resp.content.decode().split('\n')[1]


def retrieve_metar(station_icao) -> typing.Tuple[typing.Union[str, None], typing.Union[str, None]]:
    """
    Retrieves a METAR string from an online database

    Args:
        station_icao: ICAO of the station

    Returns:
        tuple of error, metar_str
    """
    url = _BASE_METAR_URL.format(station=station_icao)
    with requests.get(url) as resp:
        if not resp.ok:
            return f'unable to obtain METAR for station {station_icao}\n' \
                   f'Got to "http://tgftp.nws.noaa.gov/data/observations/metar/stations" ' \
                   f'for a list of valid stations', None
        return None, resp.content.decode().split('\n')[1]



# coding=utf-8
"""
Test coalition object
"""

import pytest

from emiz.mission import BaseUnit


def test_eq(mission):
    assert mission.blue_coa == mission.blue_coa
    with pytest.raises(ValueError):
        assert mission.blue_coa == mission.day


def test_statics(mission):
    blue_statics = list(mission.blue_coa.statics)
    red_statics = list(mission.red_coa.statics)
    assert len(blue_statics) == 3
    assert len(red_statics) == 2


def test_farps(mission):
    blue_farps = list(mission.blue_coa.farps)
    red_farps = list(mission.red_coa.farps)
    assert len(blue_farps) == len(red_farps) == 1


def get_unit_by_id(mission):
    assert mission.blue_coa.get_unit_by_id(9999) is None
    assert isinstance(mission.blue_coa.get_unit_by_id(1), BaseUnit)



# coding=utf-8
"""
Allow for batch edit of time and weather of single miz
"""

import typing

from metar.Metar import Metar

import emiz.weather
from emiz import MAIN_LOGGER
from emiz.mission_time import MissionTime
from emiz.miz import Miz

LOGGER = MAIN_LOGGER.getChild(__name__)

# pylint: disable=too-many-arguments,too-many-branches,too-many-return-statements


def edit_miz(  # noqa: C901
        infile: str,
        outfile: str = None,
        metar: typing.Union[str, Metar] = None,
        time: str = None,
        min_wind: int = 0,
        max_wind: int = 40
) -> str:
    # noinspection SpellCheckingInspection
    """
    Edit an opened MIZ file and sets the time and date and the weather

    Args:
        infile: source file
        outfile: output file (will default to source file)
        metar: metar string, ICAO or object to apply
        time: time string to apply (YYYYMMDDHHMMSS)
        min_wind: minimum wind
        max_wind: maximum wind

    Returns:
        String containing error
    """
    if outfile is None:
        LOGGER.debug(f'editing in place: {infile}')
        outfile = infile
    else:
        LOGGER.debug(f'editing miz file: {infile} -> {outfile}')

    mission_weather = mission_time = None

    if metar:
        error, metar = emiz.weather.custom_metar.CustomMetar.get_metar(metar)
        if error:
            return error

        mission_weather = emiz.weather.mission_weather.MissionWeather(metar, min_wind=min_wind, max_wind=max_wind)

    if time:
        try:
            mission_time = MissionTime.from_string(time)
        except ValueError:
            return f'badly formatted time string: {time}'

    if not mission_weather and not mission_time:
        return 'nothing to do!'

    with Miz(infile) as miz:
        if mission_weather:
            LOGGER.debug('applying MissionWeather')
            if not mission_weather.apply_to_miz(miz):
                return 'error while applying METAR to mission'
        if mission_time:
            LOGGER.debug('applying MissionTime')
            if not mission_time.apply_to_miz(miz):
                return 'error while setting time on mission'

        try:
            miz.zip(outfile)
            return ''
        except OSError:
            return f'permission error: cannot edit "{outfile}"; maybe it is in use ?'



# coding=utf-8
"""
Builds a dummy METAR string from a mission file
"""

import re
from datetime import datetime

import emiz.weather.utils
from emiz import MAIN_LOGGER
from emiz.miz import Mission, Miz

LOGGER = MAIN_LOGGER.getChild(__name__)


# pylint: disable=too-many-locals
def get_metar_from_mission(
        mission_file: str,
        icao: str = 'XXXX',
        time: str = None,
) -> str:
    """
    Builds a dummy METAR string from a mission file

    Args:
        mission_file: input mission file
        icao: dummy ICAO (defaults to XXXX)
        time: dummy time (defaults to now())

    Returns: METAR str

    """

    def _get_wind(mission_: Mission):
        wind_dir = emiz.weather.utils.reverse_direction(mission_.weather.wind_at_ground_level_dir)
        wind_speed = int(mission_.weather.wind_at_ground_level_speed)
        return f'{wind_dir:03}{wind_speed:02}MPS'

    def _get_precipitations(mission_: Mission):
        precipitations_ = {
            0: '',
            1: 'RA',
            2: 'SN',
            3: '+RA',
            4: '+SN',
        }
        return precipitations_[mission_.weather.precipitations]

    def _get_clouds(mission_: Mission):
        density = {
            0: 'CLR',
            1: 'FEW',
            2: 'FEW',
            3: 'FEW',
            4: 'SCT',
            5: 'SCT',
            6: 'SCT',
            7: 'BKN',
            8: 'BKN',
            9: 'OVC',
            10: 'OVC',
        }
        density = density[mission_.weather.cloud_density]
        base = int(round(mission_.weather.cloud_base * 3.28084, -2) / 100)
        return f'{density}{base:03}'

    def _get_temp(mission_: Mission):
        temperature = mission_.weather.temperature
        minus = 'M' if temperature < 0 else ''
        temperature = abs(temperature)
        return f'{minus}{temperature:02}/{minus}{temperature:02}'

    def _get_pressure(mission_: Mission):
        hpa = round(mission_.weather.qnh / 0.75006156130264)
        return f'Q{hpa}'

    if time is None:
        now = datetime.utcnow()
        day = now.day
        hour = now.hour
        minute = now.minute
        time = f'{day:02}{hour:02}{minute:02}Z'
    with Miz(mission_file) as miz:
        mission = miz.mission
    wind = _get_wind(mission)
    visibility = min(mission.weather.visibility, 9999)
    if mission.weather.fog_visibility > 0:
        visibility = min(mission.weather.fog_visibility, visibility)
    if visibility:
        visibility = ('{:04d}M'.format(visibility))
    precipitations = _get_precipitations(mission)
    clouds = _get_clouds(mission)
    temp = _get_temp(mission)
    pres = _get_pressure(mission)
    # noinspection SpellCheckingInspection
    qual = 'NOSIG'

    if visibility == 9999 and int(round(mission.weather.cloud_base * 3.28084, -2)) >= 5000:
        # noinspection SpellCheckingInspection
        visibility = 'CAVOK'

    metar = f'{icao} {time} {wind} {visibility} {precipitations} {clouds} {temp} {pres} {qual}'
    return re.sub(' +', ' ', metar)



# coding=utf-8
"""
Tests Mission weather
"""
import pytest


def test_qnh(mission):
    assert mission.weather.qnh == 760
    mission.weather.qnh = 754
    assert mission.weather.qnh == 754
    for wrong_qnh in ['caribou', 719, 791, -1, None, True]:
        with pytest.raises(ValueError):
            mission.weather.qnh = wrong_qnh


def test_seasons(mission):
    # Season code has been deprecated by ED
    mission.weather.season_code = mission.weather.seasons_enum['winter']
    mission.weather.temperature = -50
    assert mission.weather.temperature == -50
    mission.weather.season_code = mission.weather.seasons_enum['summer']
    assert mission.weather.temperature == 5
    # with pytest.raises(ValueError):
    mission.weather.temperature = -50
    mission.weather.temperature = 50
    assert mission.weather.temperature == 50
    mission.weather.season_code = mission.weather.seasons_enum['winter']
    assert mission.weather.temperature == 15
    # with pytest.raises(ValueError):
    mission.weather.temperature = 50
    mission.weather.temperature = -50
    mission.weather.season_code = mission.weather.seasons_enum['fall']
    assert mission.weather.temperature == -10
    mission.weather.season_code = mission.weather.seasons_enum['summer']
    assert mission.weather.temperature == 5
    mission.weather.temperature = 50
    mission.weather.season_code = mission.weather.seasons_enum['spring']
    assert mission.weather.temperature == 30
    for test in [(1, 'summer'), (2, 'winter'), (3, 'spring'), (4, 'fall')]:
        assert test[0] == mission.weather.get_season_code_from_name(test[1])
        mission.weather.season_code = test[0]
        assert mission.weather.season_name == test[1]
    for wrong_name in (1, -1, None, True, 'caribou'):
        with pytest.raises(ValueError, msg=wrong_name):
            mission.weather.get_season_code_from_name(wrong_name)


def test_wind(mission):
    assert mission.weather.wind_at2000_dir == 0
    assert mission.weather.wind_at2000_speed == 0
    assert mission.weather.wind_at8000_dir == 0
    assert mission.weather.wind_at8000_speed == 0
    assert mission.weather.wind_at_ground_level_dir == 0
    assert mission.weather.wind_at_ground_level_speed == 0
    for wrong_speed in [-1, 51, True, None, 'caribou']:
        with pytest.raises(ValueError, msg=wrong_speed):
            mission.weather.wind_at_ground_level_speed = wrong_speed
        with pytest.raises(ValueError, msg=wrong_speed):
            mission.weather.wind_at8000_speed = wrong_speed
        with pytest.raises(ValueError, msg=wrong_speed):
            mission.weather.wind_at2000_speed = wrong_speed
    for wrong_dir in [-1, 360, True, None, 'caribou']:
        with pytest.raises(ValueError, msg=wrong_dir):
            mission.weather.wind_at_ground_level_dir = wrong_dir
        with pytest.raises(ValueError, msg=wrong_dir):
            mission.weather.wind_at2000_dir = wrong_dir
        with pytest.raises(ValueError, msg=wrong_dir):
            mission.weather.wind_at8000_dir = wrong_dir
    for i in range(0, 359, 1):
        mission.weather.wind_at8000_dir = i
        mission.weather.wind_at2000_dir = i
        mission.weather.wind_at_ground_level_dir = i
    for i in range(0, 50, 1):
        mission.weather.wind_at8000_speed = i
        mission.weather.wind_at2000_speed = i
        mission.weather.wind_at_ground_level_speed = i


def test_turbulence(mission):
    assert mission.weather.turbulence_at_ground_level == 0
    for i in range(0, 60, 1):
        mission.weather.turbulence_at_ground_level = i
    for wrong_turbulence in [-1, 61, True, None, 'caribou']:
        with pytest.raises(ValueError, msg=wrong_turbulence):
            mission.weather.turbulence_at_ground_level = wrong_turbulence


def test_atmosphere_type(mission):
    assert mission.weather.atmosphere_type == 0
    mission.weather.atmosphere_type = 1
    assert mission.weather.atmosphere_type == 1
    for wrong_atmo_type in [-1, 2, True, None, 'caribou']:
        with pytest.raises(ValueError, msg=wrong_atmo_type):
            mission.weather.atmosphere_type = wrong_atmo_type


def test_fog(mission):
    assert not mission.weather.fog_enabled
    mission.weather.fog_enabled = True
    assert mission.weather.fog_enabled
    assert mission.weather.fog_visibility == 25
    assert mission.weather.fog_thickness == 0
    mission.weather.fog_visibility = 500
    assert mission.weather.fog_visibility == 500
    mission.weather.fog_thickness = 500
    assert mission.weather.fog_thickness == 500
    for i in range(0, 6000, 100):
        mission.weather.fog_visibility = i
    for i in range(0, 1000, 10):
        mission.weather.fog_thickness = i
    for wrong_visibility in [-1, 6001, True, None, 'caribou']:
        with pytest.raises(ValueError, msg=wrong_visibility):
            mission.weather.fog_visibility = wrong_visibility
    for wrong_thickness in [-1, 1001, True, None, 'caribou']:
        with pytest.raises(ValueError, msg=wrong_thickness):
            mission.weather.fog_thickness = wrong_thickness


def test_clouds(mission):
    assert mission.weather.cloud_thickness == 200
    assert mission.weather.cloud_base == 300
    for i in range(0, 10):
        mission.weather.cloud_density = i
    for wrong_density in [-1, 11, True, None, 'caribou']:
        with pytest.raises(ValueError, msg=wrong_density):
            mission.weather.cloud_density = wrong_density
    for i in range(300, 5000, 100):
        mission.weather.cloud_base = i
    for wrong_base in [-1, -500, 5001, 50000, None, False, 'caribou']:
        with pytest.raises(ValueError):
            mission.weather.cloud_base = wrong_base
    for i in range(200, 2000, 50):
        mission.weather.cloud_thickness = i
    for wrong_thickness in [199, 2001, -500, 2, 12000, None, False, 'caribou']:
        with pytest.raises(ValueError):
            mission.weather.cloud_thickness = wrong_thickness


def test_precipitations(mission):
    mission.weather.cloud_density = 4
    mission.weather.season_code = 3
    mission.weather.temperature = 1
    mission.weather.precipitations = 0
    for i in range(1, 4):
        with pytest.raises(ValueError, msg=i):
            mission.weather.precipitations = i
    mission.weather.cloud_density = 5
    mission.weather.precipitations = 1
    for i in range(2, 4):
        with pytest.raises(ValueError, msg=i):
            mission.weather.precipitations = i
    mission.weather.cloud_density = 9
    for i in range(0, 2):
        mission.weather.precipitations = i
    mission.weather.temperature = 0
    for i in range(0, 4):
        mission.weather.precipitations = i
    mission.weather.temperature = -1
    for i in range(3, 4):
        mission.weather.precipitations = i
    for i in range(1, 2):
        with pytest.raises(ValueError, msg=i):
            mission.weather.precipitations = i
    mission.weather.precipitations = 4
    # Season code has been deprecated by ED
    # miz.mission.weather.season_code = 1
    # assert miz.mission.weather.temperature == 5
    # assert miz.mission.weather.precipitations == 2
    # miz.mission.weather.season_code = 2
    # miz.mission.weather.temperature = -20
    # assert miz.mission.weather.precipitations == 4
    # miz.mission.weather.precipitations = 3
    # miz.mission.weather.season_code = 1
    # assert miz.mission.weather.precipitations == 1



# coding=utf-8
"""
Subclass metar.Metar.Metar (sic) to add functionality
"""
import typing

from metar.Metar import Metar, ParserError

from .. import noaa
from ... import MAIN_LOGGER
from .custom_metar_pressure import CustomPressure

LOGGER = MAIN_LOGGER.getChild(__name__)


class CustomMetar(Metar):
    """
    Subclass metar.Metar.Metar (sic) to add functionality
    """

    def __init__(self, metar_code, month=None, year=None, utc_delta=None):
        LOGGER.debug(f'creating METAR from: {metar_code}')
        Metar.__init__(self, metar_code, month, year, utc_delta)
        self.press = CustomPressure(self.press.value('mb'))

    @staticmethod
    def get_metar(
            metar: typing.Union[str, 'CustomMetar']
    ) -> typing.Tuple[typing.Union[str, None], typing.Union['CustomMetar', None]]:
        """
        Builds a CustomMetar object from a CustomMetar object (returns it), an ICAO code or a METAR string

        Args:
            metar: CustomMetar object, ICAO string or METAR string

        Returns: CustomMetar object

        """
        error = None
        if isinstance(metar, CustomMetar):
            return None, metar
        elif isinstance(metar, str):
            LOGGER.debug(f'building CustomMetar from: {metar}')
            if len(metar) == 4:
                LOGGER.debug('retrieving METAR from ICAO')
                error, metar = noaa.retrieve_metar(metar)
        else:
            error = f'expected a string or or a CustomMetar object, got: {type(metar)}'

        if error:
            return error, None

        try:
            return None, CustomMetar(metar_code=metar)
        except ParserError:
            return f'Unable to parse METAR: {metar}', None

    # pylint: disable=too-many-branches
    def string(self):  # noqa: C901
        """
        Return a human-readable version of the decoded report.
        """
        lines = ["station: %s" % self.station_id]
        if self.type:
            lines.append("type: %s" % self.report_type())
        if self.time:
            lines.append("time: %s" % self.time.ctime())
        if self.temp:
            lines.append("temperature: %s" % self.temp.string("C"))
        if self.dewpt:
            lines.append("dew point: %s" % self.dewpt.string("C"))
        if self.wind_speed:
            lines.append("wind: %s" % self.wind())
        if self.wind_speed_peak:
            lines.append("peak wind: %s" % self.peak_wind())
        if self.wind_shift_time:
            lines.append("wind shift: %s" % self.wind_shift())
        if self.vis:
            lines.append("visibility: %s" % self.visibility())
        if self.runway:
            lines.append("visual range: %s" % self.runway_visual_range())
        if self.press:
            lines.append(f"pressure: {self.press.string('MB')} {self.press.string('IN')} {self.press.string('MM')}")
        if self.weather:
            lines.append("weather: %s" % self.present_weather())
        if self.sky:
            lines.append("sky: %s" % self.sky_conditions("\n     "))
        if self.press_sea_level:
            lines.append("sea-level pressure: %s" % self.press_sea_level.string("mb"))
        if self.max_temp_6hr:
            lines.append("6-hour max temp: %s" % str(self.max_temp_6hr))
        if self.max_temp_6hr:
            lines.append("6-hour min temp: %s" % str(self.min_temp_6hr))
        if self.max_temp_24hr:
            lines.append("24-hour max temp: %s" % str(self.max_temp_24hr))
        if self.max_temp_24hr:
            lines.append("24-hour min temp: %s" % str(self.min_temp_24hr))
        if self.precip_1hr:
            lines.append("1-hour precipitation: %s" % str(self.precip_1hr))
        if self.precip_3hr:
            lines.append("3-hour precipitation: %s" % str(self.precip_3hr))
        if self.precip_6hr:
            lines.append("6-hour precipitation: %s" % str(self.precip_6hr))
        if self.precip_24hr:
            lines.append("24-hour precipitation: %s" % str(self.precip_24hr))
        if self._remarks:
            lines.append("remarks:")
            lines.append("- " + self.remarks("\n- "))
        if self._unparsed_remarks:
            lines.append("- " + ' '.join(self._unparsed_remarks))
        lines.append("METAR: " + self.code)
        return "\n".join(lines)

    def _handlePressure(self, d):
        """
        Parse an altimeter-pressure group.

        The following attributes are set:
            press    [int]
        """
        press = d['press']
        if press != '////':
            press = float(press.replace('O', '0'))
            if d['unit']:
                if d['unit'] == 'A' or (d['unit2'] and d['unit2'] == 'INS'):
                    self.press = CustomPressure(press / 100, 'IN')
                elif d['unit'] == 'SLP':
                    if press < 500:
                        press = press / 10 + 1000
                    else:
                        press = press / 10 + 900
                    self.press = CustomPressure(press, 'MB')
                    self._remarks.append("sea-level pressure %.1fhPa" % press)
                else:
                    self.press = CustomPressure(press, 'MB')
            elif press > 2500:
                self.press = CustomPressure(press / 100, 'IN')
            else:
                self.press = CustomPressure(press, 'MB')

    # noinspection SpellCheckingInspection
    def _handleSealvlPressRemark(self, d):
        """
        Parse the sea-level pressure remark group.
        """
        value = float(d['press']) / 10.0
        if value < 50:
            value += 1000
        else:
            value += 900
        if not self.press:
            self.press = CustomPressure(value, "MB")
        self.press_sea_level = CustomPressure(value, "MB")



# coding=utf-8
from collections import defaultdict

import click

# pylint: skip-file


@click.command()
@click.argument('miz_path', type=click.Path(exists=True, file_okay=True, dir_okay=False, readable=True))
def main(miz_path):
    from emiz.miz import Miz
    with Miz(miz_path) as m:
        mis = m.mission

    result = defaultdict(dict)
    for unit in mis.units:
        airport, spot = unit.group_name.split('#')
        spot = int(spot)
        # print(airport, int(spot), unit.unit_position)
        result[airport][spot] = unit.unit_position

    import pickle
    with open('_parking_spots.py', mode='w') as f:
        f.write('parkings = {}\n'.format(pickle.dumps(result)))

        # # print(out)
        # import src.miz.out
        # with open('out.py') as f:
        #     res = pickle.loads(src.miz.out.parkings)

        # print(res)

        # for k in result:
        #     parkings[k] = result[k]
        #
        # parkings.write()


if __name__ == '__main__':
    main()



# coding=utf-8

import random

import elib.path
import pytest
from metar.Metar import Metar

import emiz.weather
from emiz import Miz
from emiz.mission import Mission

TEST_FILES_FOLDER = elib.path.ensure_path('./test/test_files').absolute()
if not TEST_FILES_FOLDER.exists():
    raise RuntimeError(f'cannot find test files in: {str(TEST_FILES_FOLDER.absolute())}')

TEST_FILE = TEST_FILES_FOLDER.joinpath('weather.miz')
OUT_FILE = TEST_FILES_FOLDER.joinpath('weather_output.miz')
BOGUS_CLOUD_ALT_TEST_FILE = TEST_FILES_FOLDER.joinpath('TRMT_5.1.0-OVC-171220.miz')


def _randomize_weather(mission: Mission):
    mission.weather.qnh = random.randint(720, 790)
    try:
        mission.weather.precipitations = random.randint(0, 4)
    except ValueError:
        pass
    mission.weather.wind_at_ground_level_speed = random.randint(0, 50)
    mission.weather.wind_at2000_speed = random.randint(0, 50)
    mission.weather.wind_at8000_speed = random.randint(0, 50)
    mission.weather.wind_at_ground_level_dir = random.randint(0, 359)
    mission.weather.wind_at2000_dir = random.randint(0, 359)
    mission.weather.wind_at8000_dir = random.randint(0, 359)


@pytest.mark.parametrize('icao', ['UGTB', 'UGTO', 'UGKO', 'UGSA', 'UGDT', 'URSS', 'EBBR'])
def test_set_weather_from_icao(icao):
    result = emiz.weather.mizfile.set_weather_from_metar(icao, TEST_FILE, OUT_FILE)
    assert isinstance(result, tuple)
    assert result[0] is None
    with Miz(TEST_FILE) as miz:
        miz._encode()
        m1 = miz.mission
    with Miz(OUT_FILE) as miz:
        miz._encode()
        m2 = miz.mission
    for (coa1, coa2) in ((m1._red_coa, m2._red_coa), (m1._blue_coa, m2._blue_coa)):
        for attr in ('_section_bullseye', '_section_coalition', '_section_country', '_section_nav_points'):
            assert getattr(coa1, attr) == getattr(coa2, attr)
    assert m1.weather != m2.weather


@pytest.mark.parametrize('metar', [
    'UGTB 201300Z 13014KT CAVOK 33/07 Q1016 R13R/CLRD70 NOSIG',
])
def test_set_weather_from_metar(metar):
    in_metar = Metar(metar)
    random_file = elib.path.ensure_path('./test.miz', must_exist=False)
    with Miz(TEST_FILE) as miz:
        _randomize_weather(miz.mission)
        miz.zip(random_file)
    emiz.weather.mizfile.set_weather_from_metar(metar, random_file, OUT_FILE)
    out_metar = Metar(emiz.weather.mizfile.get_metar_from_mission(OUT_FILE, 'UGTB'))
    assert in_metar.temp.value() == out_metar.temp.value()
    assert int(in_metar.wind_speed.value('MPS')) == out_metar.wind_speed.value('MPS')
    assert in_metar.wind_dir.value() == out_metar.wind_dir.value()
    assert out_metar.temp.value() == 33.0
    assert out_metar.wind_dir.value() == 130
    assert out_metar.wind_speed.value() == 7.0
    with Miz(OUT_FILE) as miz:
        assert miz.mission.weather.wind_at_ground_level_dir == emiz.weather.utils.reverse_direction(
            in_metar.wind_dir.value())
        assert miz.mission.weather.visibility == 10000
        assert miz.mission.weather.cloud_thickness == 200
        assert miz.mission.weather.atmosphere_type == 0
        assert miz.mission.weather.cloud_base == 300
        assert miz.mission.weather.cloud_density == 0
        assert miz.mission.weather.turbulence_at_ground_level == 0
        assert miz.mission.weather.wind_at_ground_level_speed == 7
        assert miz.mission.weather.wind_at_ground_level_dir == 310
        assert miz.mission.weather.wind_at2000_speed >= miz.mission.weather.wind_at_ground_level_speed
        assert miz.mission.weather.wind_at8000_speed >= miz.mission.weather.wind_at2000_speed
        assert miz.mission.weather.precipitations == 0
        assert miz.mission.weather.qnh == 762


@pytest.mark.long
def test_bogus_cloud_alt():
    base = int(round(1400 * 3.28084, -2))
    assert base == 4600
    assert f'{base:04}' == '4600'
    metar = emiz.weather.mizfile.get_metar_from_mission(BOGUS_CLOUD_ALT_TEST_FILE)
    assert '02104MPS 9999M OVC046 07/07 Q1028 NOSIG' in metar
    metar = emiz.weather.custom_metar.CustomMetar(metar)
    assert 'sky: overcast at 4600 feet' in metar.string()



# coding=utf-8
"""
Subclass Metar.pressure to add MM unit (millimeter or mercury)
"""

from metar.Datatypes import UnitsError, pressure


class CustomPressure(pressure):
    """
    Subclass Metar.pressure to add MM unit (millimeter or mercury)
    """
    legal_units = ["MB", "HPA", "IN", "MM"]

    def value(self, units=None):
        """Return the pressure in the specified units."""
        if units is None:
            return self._value
        else:
            if not units.upper() in CustomPressure.legal_units:
                raise UnitsError("unrecognized pressure unit: '" + units + "'")
            units = units.upper()
        if units == self._units:
            return self._value
        if self._units == "IN":
            mb_value = self._value * 33.86398
        elif self._units == "MM":
            mb_value = self._value * 1.3332239
        else:
            mb_value = self._value
        if units == "MB" or units == "HPA":
            return mb_value
        elif units == "IN":
            return mb_value / 33.86398
        elif units == "MM":
            return mb_value / 1.3332239
        else:
            raise UnitsError("unrecognized pressure unit: '" + units + "'")

    def string(self, units=None) -> str:
        """Return a string representation of the pressure, using the given units."""
        if not units:
            units = self._units
        else:
            if not units.upper() in CustomPressure.legal_units:
                raise UnitsError("unrecognized pressure unit: '" + units + "'")
            units = units.upper()
        val = self.value(units)
        if units == "MB":
            return "%.0f mb" % val
        elif units == "HPA":
            return "%.0f hPa" % val
        elif units == "IN":
            return "%.2f inches" % val
        elif units == "MM":
            return "%.0f mmHg" % val

        raise ValueError(units)



# coding=utf-8
"""
Manages MIZ files
"""
import shutil
import tempfile
import typing
from filecmp import dircmp
from pathlib import Path
from zipfile import BadZipFile, ZipFile, ZipInfo

import elib.path

from emiz import MAIN_LOGGER
from emiz.dummy_miz import dummy_miz
from emiz.mission import Mission
from emiz.sltp import SLTP

LOGGER = MAIN_LOGGER.getChild(__name__)
ENCODING = 'iso8859_15'


# pylint: disable=too-many-instance-attributes
class Miz:
    """
    Manage MIZ files
    """

    def __init__(
            self,
            path_to_miz_file: typing.Union[str, Path],
            temp_dir: typing.Union[str, Path] = None,
            keep_temp_dir: bool = False,
            overwrite: bool = False
    ):

        self.miz_path = elib.path.ensure_file(path_to_miz_file)

        if self.miz_path.suffix != '.miz':
            raise ValueError(f'MIZ file should end with the ".miz" extension: {self.miz_path}')

        if temp_dir is not None:
            raise PendingDeprecationWarning()

        self.keep_temp_dir = keep_temp_dir

        self.overwrite = overwrite

        self.temp_dir = Path(tempfile.mkdtemp('EMFT_'))
        LOGGER.debug(f'temporary directory: {self.temp_dir}')

        self.zip_content = None
        self._mission = None
        self._mission_qual = None
        self._l10n = None
        self._l10n_qual = None
        self._map_res = None
        self._map_res_qual = None

    def __enter__(self):
        LOGGER.debug('instantiating new Mission object as a context')
        self.unzip(self.overwrite)
        self.decode()
        return self

    def __exit__(self, exc_type, exc_val, _):
        if exc_type:
            LOGGER.error(f'there were error with this mission, keeping temp dir at "{self.temp_dir}"')
            LOGGER.error('{}\n{}'.format(exc_type, exc_val))
            return False

        LOGGER.debug('closing Mission object context')
        if not self.keep_temp_dir:
            LOGGER.debug(f'removing temp dir: {self.temp_dir}')
            self._remove_temp_dir()
        return True

    @property
    def mission_file(self) -> Path:
        """

        Returns: mission file path

        """
        return self.temp_dir.joinpath('mission')

    @property
    def dictionary_file(self) -> Path:
        """

        Returns: l10n file path

        """
        return self.temp_dir.joinpath('l10n', 'DEFAULT', 'dictionary')

    @property
    def map_res_file(self) -> Path:
        """

        Returns: resource map file path

        """
        return self.temp_dir.joinpath('l10n', 'DEFAULT', 'mapResource')

    @property
    def mission(self) -> Mission:
        """

        Returns: mission

        """
        if self._mission is None:
            raise RuntimeError()
        return self._mission

    @property
    def l10n(self) -> dict:
        """

        Returns: l10n dictionary

        """
        if self._l10n is None:
            raise RuntimeError()
        return self._l10n

    @property
    def map_res(self) -> dict:
        """

        Returns: map resources dictionary

        """
        if self._map_res is None:
            raise RuntimeError()
        return self._map_res

    @staticmethod
    def reorder(
            miz_file_path: typing.Union[str, Path],
            target_dir: typing.Union[str, Path],
            skip_options_file: bool,
    ):
        """
        Re-orders a miz file into a folder (flattened)

        Args:
            miz_file_path: source miz file
            target_dir: folder to flatten the content into
            skip_options_file: do not re-order option file

        """

        miz_file_path = elib.path.ensure_file(miz_file_path)
        target_dir = elib.path.ensure_dir(target_dir, must_exist=False)

        LOGGER.debug(f're-ordering miz file: {miz_file_path}')
        LOGGER.debug(f'destination folder: {target_dir}')
        LOGGER.debug(f'{"skipping" if skip_options_file else "including"} option file')

        if not target_dir.exists():
            LOGGER.debug(f'creating directory {target_dir}')
            target_dir.mkdir(exist_ok=True)

        with Miz(miz_file_path, overwrite=True) as miz_:

            def mirror_dir(src: Path, dst: Path):
                """
                Propagates difference between the original lua tables and the re-ordered one

                Args:
                    src: source folder
                    dst: destination folder
                """
                LOGGER.debug(f'mirroring: {src} -> {dst}')

                LOGGER.debug('comparing directories')
                diff_ = dircmp(str(src), str(dst), ignore)

                diff_list = diff_.left_only + diff_.diff_files
                LOGGER.debug(f'differences: {diff_list}')

                for __diff in diff_list:
                    source = Path(diff_.left, __diff)
                    target = Path(diff_.right, __diff)
                    LOGGER.debug(f'looking at: {__diff}')
                    if source.is_dir():
                        LOGGER.debug('isdir: {}'.format(__diff))
                        if not target.exists():
                            LOGGER.debug(f'creating: {__diff}')
                            target.mkdir()
                        mirror_dir(source, target)
                    else:
                        LOGGER.debug(f'copying: {__diff}')
                        shutil.copy2(str(source), diff_.right)
                for sub in diff_.subdirs.values():
                    assert isinstance(sub, dircmp)
                    mirror_dir(sub.left, sub.right)

            # pylint: disable=protected-access
            miz_._encode()

            if skip_options_file:
                ignore = ['options']
            else:
                ignore = []

            mirror_dir(miz_.temp_dir, target_dir)

    def decode(self):
        """Decodes the mission files into dictionaries"""

        LOGGER.debug('decoding lua tables')

        if not self.zip_content:
            self.unzip(overwrite=False)

        LOGGER.debug('reading map resource file')
        with open(self.map_res_file, encoding=ENCODING) as stream:
            self._map_res, self._map_res_qual = SLTP().decode(stream.read())

        LOGGER.debug('reading l10n file')
        with open(self.dictionary_file, encoding=ENCODING) as stream:
            self._l10n, self._l10n_qual = SLTP().decode(stream.read())

        LOGGER.debug('reading mission file')
        with open(self.mission_file, encoding=ENCODING) as stream:
            mission_data, self._mission_qual = SLTP().decode(stream.read())
            self._mission = Mission(mission_data, self._l10n)

        LOGGER.debug('decoding done')

    def _encode(self):

        LOGGER.debug('encoding lua tables')

        LOGGER.debug('encoding map resource')
        with open(self.map_res_file, mode='w', encoding=ENCODING) as stream:
            stream.write(SLTP().encode(self._map_res, self._map_res_qual))

        LOGGER.debug('encoding l10n dictionary')
        with open(self.dictionary_file, mode='w', encoding=ENCODING) as stream:
            stream.write(SLTP().encode(self.l10n, self._l10n_qual))

        LOGGER.debug('encoding mission dictionary')
        with open(self.mission_file, mode='w', encoding=ENCODING) as stream:
            stream.write(SLTP().encode(self.mission.d, self._mission_qual))

        LOGGER.debug('encoding done')

    def _check_extracted_content(self):

        for filename in self.zip_content:
            path = Path(self.temp_dir.joinpath(filename))
            if not path.exists():
                raise FileNotFoundError(str(path))

    def _extract_files_from_zip(self, zip_file):

        for item in zip_file.infolist():  # not using ZipFile.extractall() for security reasons
            assert isinstance(item, ZipInfo)

            LOGGER.debug(f'unzipping item: {item.filename}')

            try:
                zip_file.extract(item, str(self.temp_dir))
            except:  # noqa: E722
                LOGGER.error(f'failed to extract archive member: {item.filename}')
                raise

    def _remove_temp_dir(self):
        shutil.rmtree(str(self.temp_dir))

    def unzip(self, overwrite: bool = False):
        """
        Flattens a MIZ file into the temp dir

        Args:
            overwrite: allow overwriting exiting files

        """

        if self.zip_content and not overwrite:
            raise FileExistsError(str(self.temp_dir))

        LOGGER.debug('unzipping miz to temp dir')

        try:

            with ZipFile(str(self.miz_path)) as zip_file:

                LOGGER.debug('reading infolist')

                self.zip_content = [f.filename for f in zip_file.infolist()]

                self._extract_files_from_zip(zip_file)

        except BadZipFile:
            raise BadZipFile(str(self.miz_path))

        except:  # noqa: E722
            LOGGER.exception(f'error while unzipping miz file: {self.miz_path}')
            raise

        LOGGER.debug('checking miz content')

        # noinspection PyTypeChecker
        for miz_item in ['mission', 'options', 'warehouses', 'l10n/DEFAULT/dictionary', 'l10n/DEFAULT/mapResource']:
            if not Path(self.temp_dir.joinpath(miz_item)).exists():
                LOGGER.error('missing file in miz: {}'.format(miz_item))
                raise FileNotFoundError(miz_item)

        self._check_extracted_content()

        LOGGER.debug('all files have been found, miz successfully unzipped')

    def zip(self, destination: typing.Union[str, Path] = None) -> str:
        """
        Write mission, dictionary etc. to a MIZ file

        Args:
            destination: target MIZ file (if none, defaults to source MIZ + "_EMIZ"

        Returns: destination file

        """

        self._encode()

        if destination is None:
            destination = self.miz_path.parent.joinpath(f'{self.miz_path.stem}_EMIZ.miz')
        else:
            destination = elib.path.ensure_file(destination, must_exist=False)

        LOGGER.debug('zipping mission to: {}'.format(destination))

        destination.write_bytes(dummy_miz)

        with ZipFile(str(destination), mode='w', compression=8) as zip_file:

            for zip_content in self.zip_content:
                file = Path(self.temp_dir.joinpath(zip_content))
                LOGGER.debug(f'injecting in zip file: {file}')
                zip_file.write(str(file), arcname=zip_content)

        return str(destination)



# coding=utf-8
"""
This is a temporary test to cover the import of modules that have 0% coverage so far

Yes, this is shameful...
"""

import glob

import pytest

import emiz


@pytest.fixture(autouse=True)
def patch_version(monkeypatch):
    monkeypatch.setattr(emiz, '__version__', '0.0.0')


# noinspection PyUnresolvedReferences,PyProtectedMember
@pytest.mark.nocleandir
@pytest.mark.parametrize('module', glob.glob('./emiz/*.py', recursive=True))
def test_imports(module):
    module = module[2:-3].replace('\\', '.')
    print(f'importing {module}')
    __import__(module)


# noinspection PyUnresolvedReferences,PyProtectedMember
@pytest.mark.nocleandir
@pytest.mark.parametrize('module', list(glob.glob('./test/*.py', recursive=True)))
def test_imports_tests(module):
    module = module[2:-3].replace('\\', '.')
    print(f'importing {module}')
    __import__(module)



# coding=utf-8
"""
Utility function for weather
"""

from emiz import MAIN_LOGGER

LOGGER = MAIN_LOGGER.getChild(__name__)


# noinspection SpellCheckingInspection
def hpa_to_mmhg(pressure: int) -> int:
    """
    Converts pressure in hpa to mmhg
    Args:
        pressure: pressure to convert

    Returns: pressure in mmhg

    """
    return int(pressure * 0.75006156130264)


def reverse_direction(heading: int) -> int:
    """
    Inverts a direction in degrees

    Args:
        heading: original direction

    Returns: direction plus or minus 180°

    """
    if heading >= 180:
        return int(heading - 180)

    return int(heading + 180)



# coding=utf-8
"""
Tests BaseMissionObject
"""

import pytest

from emiz.mission import BaseMissionObject, BaseUnit, Group


def test_init():
    with pytest.raises(TypeError):
        BaseMissionObject(None, {})
    with pytest.raises(TypeError):
        BaseMissionObject({}, None)
    BaseMissionObject({}, {})


def test_get_country_by_name(mission):
    with pytest.raises(ValueError):
        mission.get_country_by_name('nope')
    country = mission.get_country_by_name('USA')
    assert country is mission.get_country_by_name('USA')


def test_get_country_by_id(mission):
    with pytest.raises(ValueError):
        mission.get_country_by_id(9999)
    country = mission.get_country_by_id(1)
    assert country is mission.get_country_by_id(1)


@pytest.mark.parametrize('category', ['helicopter', 'ship', 'plane', 'vehicle'])
def test_get_groups_from_category(mission, category):
    with pytest.raises(ValueError):
        list(mission.get_groups_from_category(None))
    assert isinstance(list(mission.get_groups_from_category(category)), list)


@pytest.mark.parametrize('category', ['helicopter', 'ship', 'plane', 'vehicle'])
def test_get_units_from_category(mission, category):
    with pytest.raises(ValueError):
        list(mission.get_units_from_category(None))
    assert isinstance(list(mission.get_units_from_category(category)), list)


def test_get_group_by_id(mission):
    with pytest.raises(ValueError):
        mission.get_group_by_id(None)
    assert mission.get_group_by_id(99999) is None
    assert isinstance(mission.get_group_by_id(1), Group)


def test_get_unit_by_id(mission):
    with pytest.raises(ValueError):
        mission.get_unit_by_id(None)
    assert mission.get_unit_by_id(99999) is None
    assert isinstance(mission.get_unit_by_id(1), BaseUnit)


def test_get_clients_groups(mission):
    result = list(mission.get_clients_groups())
    assert len(result) == 3
    names = ['etcher', 'gal', 'gilles']
    for group in result:
        assert isinstance(group, Group)
        assert group.group_name in names


def test_day(mission):
    assert mission.day == 1
    mission.month = 2
    mission.day = 31
    assert mission.day == 28


@pytest.mark.parametrize('day', [-1, 100, None, '1'])
def test_wrong_day(mission, day):
    with pytest.raises(ValueError):
        mission.day = day


def test_month(mission):
    assert mission.month == 6
    mission.month = 2
    assert mission.month == 2


@pytest.mark.parametrize('month', [-1, 13, None, '1'])
def test_wrong_month(mission, month):
    with pytest.raises(ValueError):
        mission.month = month


def test_year(mission):
    assert mission.year == 2011
    mission.year = 2012
    assert mission.year == 2012


@pytest.mark.parametrize('year', [-1, 1899, 2101, 13, None, '1'])
def test_wrong_year(mission, year):
    with pytest.raises(ValueError):
        mission.year = year



# coding=utf-8

import pytest

from emiz.weather import AVWX, AVWXResult


def test_query_icao_ugtb():
    data = AVWX.query_icao('UGTB')
    assert isinstance(data, AVWXResult)
    assert isinstance(data.altimeter, str)
    assert data.info['Elevation'] == '495'
    assert data.info['City'] == 'Tbilisi'
    assert data.info['Country'] == 'GEO'
    assert data.info['ICAO'] == data.station == 'UGTB'
    assert data.info['Name'] == 'International Airport'

    str_data = ['altimeter', 'dewpoint', 'flightrules', 'rawreport', 'remarks',
                'speech', 'summary', 'temperature', 'time', 'visibility', 'winddirection',
                'windgust', 'windspeed']
    dict_data = ['remarksinfo', 'units']
    list_data = ['windvariabledir', 'otherlist', 'runwayvislist', 'cloudlist']

    for x in str_data:
        assert isinstance(getattr(data, x), str), x

    for x in list_data:
        assert isinstance(getattr(data, x), list), x

    for x in dict_data:
        assert isinstance(getattr(data, x), dict), x


@pytest.mark.parametrize('icao', ['UGTB', 'UGKO', 'KJFK', 'LFPG', 'URSS', 'EBBR'])
def test_query_icao(icao):
    data = AVWX.query_icao(icao)
    assert isinstance(data, AVWXResult)
    assert isinstance(data.altimeter, str)

    str_data = ['altimeter', 'dewpoint', 'flightrules', 'rawreport', 'remarks',
                'speech', 'summary', 'temperature', 'time', 'visibility', 'winddirection',
                'windgust', 'windspeed']
    dict_data = ['remarksinfo', 'units']
    list_data = ['windvariabledir', 'otherlist', 'runwayvislist', 'cloudlist']

    for x in str_data:
        assert isinstance(getattr(data, x), str), x

    for x in list_data:
        assert isinstance(getattr(data, x), list), x

    for x in dict_data:
        assert isinstance(getattr(data, x), dict), x


def test_speech():
    speech = AVWX.metar_to_speech('KJFK 241051Z 35007KT 10SM SCT200 BKN250 03/M02 A3016 RMK AO2 SLP211 T00281017')
    assert 'Winds three five zero at 7kt. Visibility one zero ' \
           'miles. Temperature three degrees Celsius. Dew point minus two degrees Celsius. Q N H three zero ' \
           'point one six. Scattered clouds at 20000ft. Broken layer at 25000ft' == speech



# coding=utf-8

import os
from pathlib import Path

import pytest

from emiz.mission import Mission
from emiz.miz import Miz

if os.path.exists('./test_files'):
    BASE_PATH = os.path.abspath('./test_files')
elif os.path.exists('./test/test_files'):
    BASE_PATH = os.path.abspath('./test/test_files')
else:
    raise RuntimeError('cannot find test files')

TEST_FILE = os.path.join(BASE_PATH, 'TRG_KA50.miz')
OUT_FILE = '{}_EMFT.miz'.format(TEST_FILE[:-4])
BAD_ZIP_FILE = os.path.join(BASE_PATH, 'bad_zip_file.miz')
MISSING_FILE = os.path.join(BASE_PATH, 'missing_files.miz')
ALL_OBJECTS = os.path.join(BASE_PATH, 'all_objects.miz')
BAD_FILES = ['bad_zip_file.miz', 'missing_files.miz']


class TestMizPath:
    @pytest.mark.parametrize('cls', [Miz])
    def test_init(self, tmpdir, cls):
        t = Path(str(tmpdir))

        f = t.joinpath('f.txt')

        with pytest.raises(FileNotFoundError):
            cls(f)

        with pytest.raises(TypeError):
            cls(t)

        f.write_text('')

        with pytest.raises(ValueError):
            cls(f)

        f = t.joinpath('f.miz')
        f.write_text('')

        cls(f)

    def test_unzip(self):
        m = Miz(TEST_FILE)
        m.unzip()

    def test_context(self):
        with Miz(TEST_FILE) as miz:
            assert isinstance(miz.mission, Mission)
            tmpdir = os.path.abspath(miz.temp_dir)

        assert not os.path.exists(tmpdir)
